@@ -32,6 +32,7 @@ extern char *safeban(char *);
 extern char *getfirsthostless(char *);
 extern char *isnickbk(char *, char *);
 extern char *getkickreason(char *);
+extern char *clonechannel(char *, char *);
 extern int topiccount(char *);
 extern int settopic(void);
 extern int setnewtopic(char *, char *, char *);
@@ -86,7 +87,7 @@ extern int ismoderated(char *);
 extern int permbancount(char *);
 extern int nthmode(char *, int);
 extern int userleft(char *, char *, char *);
-extern int changeuser(char *, char *, int, int, int);
+extern int changeuser(char *, char *, int, int, int, int);
 extern int whichctcp(char *);
 extern int whichcommand(char *, int);
 extern int gettobek(char *, char *);
@@ -117,6 +118,8 @@ extern int delserver(char *, int, int, int, int, int);
 extern int setpassword(char *, char *, char *);
 extern int logoff(char *, char *);
 extern int logon(char *, char *, char *, char *, int);
+extern int timed_cmp(struct timed *, struct timed *);
+extern int isfriend(char *, char *, char *);
 extern struct authed *addnewauthed(char *, time_t, struct botuser *, char *);
 extern struct botuser *addbotuser(char *, char *);
 extern struct botuser *addnewadmin(char *, char *);
@@ -137,13 +140,12 @@ extern struct maskstruct *addnewkickreason(char *);
 extern struct maskstruct *addnewusermask(char *, char *, char *);
 extern struct maskstruct *addnewadminmask(char *, char *);
 extern struct maskstruct *editmask(struct maskstruct *, char *, char *);
-extern struct modeline *addnewmm(char *, time_t, char *, char *);
 extern struct pubmsg *addnewpubmsg(char *, time_t, char *, char *, char *);
 extern struct part *addnewpart(char *, time_t, char *, char *);
 extern struct join *addnewjoin(char *, time_t, char *, char *);
 extern struct timed *addnewtimed(time_t, int, int, char *, char *, char *, char *);
 extern struct chanuser *getrandomuser(char *);
-extern struct ls_wrapstruct *addnew_ls(struct ls_wrapstruct *, time_t, char *, char *, char *);
+extern struct timed *timed_new(time_t, int, int, char *, char *);
 extern time_t lastoff(char *, char *, char *);
 extern void sendadverts(char *, char *, char *);
 extern void resetparts(char *, char *, char *);
@@ -155,8 +157,6 @@ extern void unlockchan(char *, char *, char *);
 extern void clearflood(char *, char *, char *);
 extern void initchannel(char *);
 extern void initall(void);
-extern void inittimeds(void);
-extern void initmmodes(char *);
 extern void initnicks(char *);
 extern void initpubmsgs(char *);
 extern void initparts(char *);
@@ -210,3 +210,4 @@ extern void nstats(char *, int, char *, char *);
 extern void wall(char *, char *, char *, char *);
 extern void cleanautheds(char *);
 extern void listchanusers(void);
+extern void timed_del(struct timed *);
