@@ -33,7 +33,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 int serverconnection(void);
 int registerconnection(void);
 int handleserverdata(void);
-int processmmodes(void);
 int processtimeds(void);
 void reconnect(void);
 
@@ -194,6 +193,8 @@ int serverconnection()
 int registerconnection()
 {
 	char stringbuffer[WRITE_BUFFER_LENGTH + 1];
+	char newuser[USERLEN + 1];
+	char *ptr;
 	struct passwd *pwd;
 
 	/* check if connected */
@@ -228,10 +229,17 @@ int registerconnection()
 		if(!mystate->customuser) {
 			if((pwd = getpwuid(getuid())) == NULL) {
 				return OER_REGISTERCONNECTION_ERR_GETPWUID;
-			} else {
-				strncpy(mystate->user, pwd->pw_name, USERLEN);
 			}
+			strncpy(mystate->user, pwd->pw_name, USERLEN);
 		}
+		ptr = mystate->user;
+                while(*ptr == '~') {
+                        /* server has prefixed our ident with ~
+                           strip ~ before saving to avoid double ~ */
+                        ptr++;
+                }
+                strncpy(newuser, ptr, USERLEN);
+                strncpy(mystate->user, newuser, USERLEN);
 		snprintf(stringbuffer, WRITE_BUFFER_LENGTH, "USER %s %s %s :%s\n", mystate->user, mystate->host, mystate->current_server->serverhost, mystate->realname);
 		oer_debug(OER_DEBUG_INFO, "registerconnection->%s", stringbuffer);
 		if(write(mystate->sockfd, stringbuffer, strlen(stringbuffer)) < 0) {
@@ -307,182 +315,69 @@ int handleserverdata()
         return OER_HANDLESERVERDATA_EVERYTHING_OK;
 }
 
-int processmmodes()
-{
-	int mmodes;
-	int actions;
-	char mode[CHANLEN + 1];
-	char target[HUGESTRINGLEN + 1];
-	char stringbuffer[WRITE_BUFFER_LENGTH + 1];
-	struct channel *this;
-	struct modeline *mm;
-	struct modeline *tm;
-
-	/* check if connected */
-	if(mystate->current_server == NULL) {
-		return OER_PROCESSMMODES_EVERYTHING_OK;
-	}
-	/* check if registered */
-	if(mystate->current_server->registered != OER_REGISTERCONNECTION_STATUS_DONE) {
-		return OER_PROCESSMMODES_EVERYTHING_OK;
-	}
-	if(mystate->netjoining) {
-		/* don't do any actions while net is joining */
-		return OER_PROCESSMMODES_EVERYTHING_OK;
-	}
-        for(this = mystate->channels, actions = 0; this != NULL; this = this->next) {
-		if(actions >= OER_ACTIONS_PER_MAINLOOP) {
-			return OER_PROCESSMMODES_EVERYTHING_OK;
-		}
-		if(!this->joined) {
-			/* don't process any modes on channels we
-			   aren't on */
-			continue;
-		}
-		/* we like to play on the safe side */
-		memset(&mode, 0, CHANLEN + 1);
-		memset(&target, 0, HUGESTRINGLEN + 1);
-		mmodes = 0;
-		mm = this->modelines;
-		while(mm != NULL && mmodes < mystate->current_server->servermodes) {
-			tm = mm->next;
-			if(mystate->now >= mm->at && !mm->done) {
-				strcat(mode, mm->mode);
-				strcat(target, mm->target);
-				strcat(target, " ");
-				mmodes++;
-				mm->done = 1;
-				/* only 1 mmode special case */
-				if(mm->prev == NULL && mm->next == NULL) {
-					this->modelines = NULL;
-					free(mm->mode);
-					free(mm->target);
-					free(mm);
-					tm = NULL;
-				} else {
-					/* the normal case, >1 mmodes */
-					if(mm->prev == NULL) {
-						/* 1st mmode */
-						this->modelines = mm->next;
-						mm->next->prev = NULL;
-						tm = mm->next;
-					} else {
-						if(mm->next == NULL) {
-							/* last mmode */
-							mm->prev->next = NULL;
-							tm = mm->prev;
-						} else {
-							/* between 1 or more mmodes */
-							mm->prev->next = mm->next;
-							mm->next->prev = mm->prev;
-							tm = mm->next;
-						}
-					}
-					free(mm->mode);
-					free(mm->target);
-					free(mm);
-				}
-			}
-			mm = tm;
-		}
-		if(mmodes) {
-			snprintf(stringbuffer, WRITE_BUFFER_LENGTH, "MODE %s %s %s\n", this->name, mode, target);
-			oer_debug(OER_DEBUG_INFO, "processmmodes->%s", stringbuffer);
-			if(write(mystate->sockfd, stringbuffer, strlen(stringbuffer)) < 0) {
-				oer_debug(OER_DEBUG_ERROR, "processmodes->write failed: %s\n", strerror(errno));
-				return OER_PROCESSMMODES_ERR_WRITE;
-			}
-			actions++;
-			mystate->current_server->lastping = mystate->now;
-		}
-	}
-	return OER_PROCESSMMODES_EVERYTHING_OK;
-}
-
 int processtimeds()
 {
 	int got_timed;
-	int actions;
-	struct channel *this;
-	struct timed *tim;
-	struct timed *tim2;
-	char stringbuffer[WRITE_BUFFER_LENGTH + 1];
+        int actions;
+        struct channel *this;
+        struct timed *t1;
+        struct timed *t2;
+        char stringbuffer[WRITE_BUFFER_LENGTH + 1];
 
-	/* check if connected */
-	if(mystate->current_server == NULL) {
-		return OER_PROCESSTIMEDS_EVERYTHING_OK;
-	}
-	/* check if registered */
-	if(mystate->current_server->registered != OER_REGISTERCONNECTION_STATUS_DONE) {
-		return OER_PROCESSTIMEDS_EVERYTHING_OK;
-	}
-	if(mystate->netjoining) {
-		/* don't do any actions while net is joining */
-		return OER_PROCESSTIMEDS_EVERYTHING_OK;
-	}
-	for(tim = mystate->timeds, actions = 0; tim != NULL; tim = tim2) {
-		tim2 = tim->next;
-		if(tim->at > mystate->now) {
-			continue;
-		}
-		if(actions >= OER_ACTIONS_PER_MAINLOOP) {
-			return OER_PROCESSTIMEDS_EVERYTHING_OK;
-		}
-		got_timed = 0;
-		switch(tim->type) {
-		case OER_TIMED_IRC_COMMAND:
-			switch(tim->function) {
-			case OER_TIMED_IRC_COMMAND_KICK:
-				if((this = getchptr(tim->optstring1)) == NULL) {
-					return OER_PROCESSTIMEDS_EVERYTHING_OK;
-				}
-				/* this check is necessary because KICK messages are
-				   scheduled before executed and it is possible that
-				   there was manual prevention (!unlock) */
-				if(!this->locked.unlocked) {
-					strncpy(stringbuffer, tim->irc_command, WRITE_BUFFER_LENGTH);
-					got_timed = 1;
-				}
-				break;
-			case OER_TIMED_IRC_COMMAND_DUMMY:
-				strncpy(stringbuffer, tim->irc_command, WRITE_BUFFER_LENGTH);
-				got_timed = 1;
-				break;
-			}
-			break;
-		}
-		if(got_timed) {
-			oer_debug(OER_DEBUG_INFO, "processtimeds->%s", stringbuffer);
-			if(write(mystate->sockfd, stringbuffer, strlen(stringbuffer)) < 0) {
-				oer_debug(OER_DEBUG_ERROR, "processtimeds->write failed: %s\n", strerror(errno));
-				return OER_PROCESSTIMEDS_ERR_WRITE;
-			}
-			actions++;
-			mystate->current_server->lastping = mystate->now;
-		}
-		/* adjust lists */
-                if(tim->prev == NULL) {
-                        /* first timed in list */
-                        mystate->timeds = tim->next;
-                        /* ... check if also last */
-                        if(tim->next != NULL) {
-                                tim->next->prev = NULL;
+        /* check if connected */
+        if(mystate->current_server == NULL) {
+                return OER_PROCESSTIMEDS_EVERYTHING_OK;
+        }
+        /* check if registered */
+        if(mystate->current_server->registered != OER_REGISTERCONNECTION_STATUS_DONE) {
+                return OER_PROCESSTIMEDS_EVERYTHING_OK;
+        }
+        if(mystate->netjoining) {
+                /* don't do any actions while net is joining */
+                return OER_PROCESSTIMEDS_EVERYTHING_OK;
+        }
+	for(t1 = mystate->timeds, actions = 0; t1 != NULL; t1 = t2) {
+                if(t1->at > mystate->now) {
+                        /* DO NOT REMOVE */
+                        t2 = t1->next;
+                        continue;
+                }
+                if(actions >= OER_ACTIONS_PER_MAINLOOP) {
+                        return OER_PROCESSTIMEDS_EVERYTHING_OK;
+                }
+                got_timed = 0;
+                switch(t1->type) {
+                case OER_TIMED_TYPE_KICK:
+                        if((this = getchptr(t1->channel)) == NULL) {
+                                return OER_PROCESSTIMEDS_EVERYTHING_OK;
                         }
-                }
-                if(tim->next == NULL) {
-                        /* last timed in list */
-                        /* ... check also if the only one */
-                        if(tim->prev != NULL) {
-                                tim->prev->next = NULL;
+                        /* this check is necessary because KICK messages are
+                           scheduled before executed and it is possible that
+                           there was manual prevention (!unlock) */
+                        if(!this->locked.unlocked) {
+                                strncpy(stringbuffer, t1->command, WRITE_BUFFER_LENGTH);
+                                got_timed = 1;
                         }
+                        break;
+                case OER_TIMED_TYPE_NORMAL:
+                        strncpy(stringbuffer, t1->command, WRITE_BUFFER_LENGTH);
+                        got_timed = 1;
+                        break;
                 }
-                if(tim->prev != NULL && tim->next != NULL) {
-                        /* between 2 or more timed */
-                        tim->prev->next = tim->next;
-                        tim->next->prev = tim->prev;
+                if(got_timed) {
+                        oer_debug(OER_DEBUG_INFO, "processtimeds->%s", stringbuffer);
+                        if(write(mystate->sockfd, stringbuffer, strlen(stringbuffer)) < 0) {
+                                oer_debug(OER_DEBUG_ERROR, "processtimeds->write failed: %s\n", strerror(errno));
+                                return OER_PROCESSTIMEDS_ERR_WRITE;
+                        }
+                        actions++;
+                        mystate->current_server->lastping = mystate->now;
                 }
-	}
-	return OER_PROCESSTIMEDS_EVERYTHING_OK;
+                t2 = t1->next;
+                timed_del(t1);
+                t1 = t2;
+        }
+        return OER_PROCESSTIMEDS_EVERYTHING_OK;
 }
 
 
