@@ -157,25 +157,10 @@ enum WHICHCOMMAND_RETURN_VALUES {
 	OER_WHICHCOMMAND_LOGON,
 	OER_WHICHCOMMAND_LOGOFF,
 	OER_WHICHCOMMAND_WALL,
+        OER_WHICHCOMMAND_CLONECHANNEL,
 	OER_WHICHCOMMAND_INVALID
 };
 
-enum TIMED_TYPES {
-	OER_TIMED_IRC_COMMAND,
-	OER_TIMED_INTERNAL_COMMAND
-};
-
-enum TIMED_FUNCTION {
-	OER_TIMED_IRC_COMMAND_KICK,
-	OER_TIMED_IRC_COMMAND_DUMMY,
-	OER_TIMED_INTERNAL_COMMAND_DUMMY
-};
-
-enum PROCESSMMODES_RETURN_VALUES {
-	OER_PROCESSMMODES_ERR_WRITE,
-	OER_PROCESSMMODES_EVERYTHING_OK
-};
-
 enum PROCESSTIMEDS_RETURN_VALUES {
 	OER_PROCESSTIMEDS_ERR_WRITE,
 	OER_PROCESSTIMEDS_EVERYTHING_OK
@@ -241,6 +226,22 @@ enum OER_BAN_TYPES {
         OER_BAN_TYPE_INVALID
 };
 
+enum OER_TIMED_TYPES {
+        OER_TIMED_TYPE_NORMAL = 1,
+        OER_TIMED_TYPE_KICK,
+        OER_TIMED_TYPE_INVALID
+};
+
+enum OER_TIMED_PRIORITIES {
+        OER_TIMED_PRIORITY_FLOOD = 1,
+        OER_TIMED_PRIORITY_NOTICE,
+        OER_TIMED_PRIORITY_PRIVMSG,
+        OER_TIMED_PRIORITY_NORMAL,
+        OER_TIMED_PRIORITY_CHANNEL_HANDLING,
+        OER_TIMED_PRIORITY_CHANNEL_PROTECTION,
+        OER_TIMED_PRIORITY_CHANNEL_INVALID
+};
+
 #define numofparams(x) (wordcount(x) - 1)
 
 /* from glib.h */
@@ -250,7 +251,7 @@ enum OER_BAN_TYPES {
 #define OER_MAINLOOP_TIMEOUT_SECS 1
 #define OER_MAINLOOP_TIMEOUT_USECS 0
 /* only change this if you are 100% sure what you are doing */
-#define OER_ACTIONS_PER_MAINLOOP 1
+#define OER_ACTIONS_PER_MAINLOOP 2
 #define OER_NICKS_PER_USERHOST 5
 /* multiple kicks aren't supported for backwards compatibility */
 #define OER_NICKS_PER_KICK 1
@@ -276,6 +277,7 @@ enum OER_BAN_TYPES {
 #define OER_NETJOIN_DELAY 15
 #define OER_PUBMSGS 20
 #define OER_SERVER_IS_STONED_FOR 300
+#define OER_LINENOISE_INTERVAL_MIN 1
 #define OER_MASSMODE_LIMIT 3
 #define OER_LAST 3
 #define OER_LAST_MAX 10
@@ -325,7 +327,7 @@ enum OER_BAN_TYPES {
 #define OER_HELP "For a complete command reference go to http://oer.equnet.org/help.html"
 #define DEFAULT_NICKBK_MESSAGE "prohibited nick"
 #define OER_REALNAME "http://oer.equnet.org/help.html"
-#define OER_COPYRIGHT1 "oer+MySQL version 1.0-17, Copyright (C) 2000-2001 EQU <equ@equnet.org>"
+#define OER_COPYRIGHT1 "oer+MySQL version 1.0-18, Copyright (C) 2000-2001 EQU <equ@equnet.org>"
 #define OER_COPYRIGHT2 "oer+MySQL comes with ABSOLUTELY NO WARRANTY; for details see the\n\
 GNU General Public License (COPYING) provided with this distribution.\n\
 This is free software, and you are welcome to redistribute it under\n\
@@ -374,6 +376,7 @@ struct chanuser {
 	int chanop;
 	int voice;
 	int tobek;
+	int friend;
 	struct chanuser *prev;
 	struct chanuser *next;
 };
@@ -395,27 +398,6 @@ struct authed {
 	struct authed *next;
 };
 
-struct modeline {
-	int done;
-	time_t at;
-	char *mode;
-	char *target;
-	struct modeline *prev;
-	struct modeline *next;
-};
-
-struct timed {
-	time_t at;
-	int type;
-	int function;
-	char *irc_command;
-	char *optstring1;
-	char *optstring2;
-	char *optstring3;
-	struct timed *prev;
-	struct timed *next;
-};
-
 struct pubmsg {
 	time_t at;
 	char *nick;
@@ -442,6 +424,16 @@ struct part {
 	struct part *next;
 };
 
+struct timed {
+        time_t at;
+        int type;
+        int prio;
+        char *channel;
+        char *command;
+        struct timed *prev;
+        struct timed *next;
+};
+
 struct maskstruct {
 	char *mask;
 	char *optstring;
@@ -494,7 +486,6 @@ struct channel {
 	struct join *joins;
 	struct part *parts;
 	struct pubmsg *pubmsgs;
-	struct modeline *modelines;
 	struct locked locked;
         struct floodvars floodvars;
         struct banvars banvars;
@@ -539,6 +530,7 @@ struct state {
 	time_t startup;
 	time_t postnj_checks_at;
 	time_t admints;
+	time_t last_linenoise;
 	struct botuser *admins;
 	struct channel *channels;
 	struct maskstruct *trusted;
