@@ -155,7 +155,7 @@ void parseirc(char *serveroutput)
 				/* isoper */
 				for(this = mystate->channels; this != NULL; this = this->next) {
 					if(isonchan(this->name, nick)) {
-						changeuser(this->name, nick, 1, -1, -1);
+						changeuser(this->name, nick, 1, -1, -1, -1);
 					}
 				}
 			}
@@ -364,7 +364,7 @@ void parseirc(char *serveroutput)
 			return;
 		}
 		oer_debug(OER_DEBUG_INFO, "parseirc(484)->nick %s on %s is a IRC operator\n", nick, channel);
-		changeuser(channel, nick, 1, -1, -1);
+		changeuser(channel, nick, 1, -1, -1, -1);
 		return;
 	}
 	if(!strcasecmp(command, "513")) {
@@ -448,13 +448,16 @@ void parseirc(char *serveroutput)
 			}
 		}
 		if(amiop(channel) && !kicked && haschanflags(channel, "v") && hasuserflags(channel, userhost, "v")) {
-			addnewmm(channel, mystate->now,  "+v", nick);
+			snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s +v %s\n", channel, nick);
+			timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 		}
 		if(amiop(channel) && !kicked && haschanflags(channel, "O") && isopa(userhost)) {
-                        addnewmm(channel, mystate->now, "+o", nick);
+			snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s +o %s\n", channel, nick);
+			timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
                 }
 		if(amiop(channel) && !kicked && haschanflags(channel, "o") && isop(channel, userhost)) {
-			addnewmm(channel, mystate->now, "+o", nick);
+			snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s +o %s\n", channel, nick);
+			timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 		}
 		if(!kicked && haschanflags(channel, "A")) {
                         sendadverts(channel, nick, userhost);
@@ -506,8 +509,8 @@ void parseirc(char *serveroutput)
 			}
 			if(usersonchan(this->name) == 1 && !amiop(this->name)) {
 				oer_debug(OER_DEBUG_INFO, "parseirc(QUIT)->cycling %s to gain ops\n", this->name);
-				snprintf(outstring2, WRITE_BUFFER_LENGTH, "PART %s\n", this->name);
-				addnewtimed(mystate->now, OER_TIMED_IRC_COMMAND, OER_TIMED_IRC_COMMAND_DUMMY, outstring2, NULL, NULL, NULL);
+				snprintf(timed_str, WRITE_BUFFER_LENGTH, "PART %s\n", this->name);
+				timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_HANDLING, NULL, timed_str);
 			}
 		}
 		listchanusers();
@@ -533,8 +536,8 @@ void parseirc(char *serveroutput)
 		}
 		if(usersonchan(channel) == 1 && !amiop(channel)) {
 			oer_debug(OER_DEBUG_INFO, "parseirc(PART)->cycling %s to gain ops\n", channel);
-			snprintf(outstring, STRINGLEN, "PART %s\n", channel);
-			addnewtimed(mystate->now, OER_TIMED_IRC_COMMAND, OER_TIMED_IRC_COMMAND_DUMMY, outstring, NULL, NULL, NULL);
+			snprintf(timed_str, WRITE_BUFFER_LENGTH, "PART %s\n", channel);
+			timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_HANDLING, NULL, timed_str);
 		}
 		listchanusers();
 		return;
@@ -603,8 +606,8 @@ void parseirc(char *serveroutput)
 		}
 		if(usersonchan(channel) == 1 && !amiop(channel)) {
 			oer_debug(OER_DEBUG_INFO, "parseirc(QUIT)->cycling %s to gain ops\n", channel);
-			snprintf(outstring2, WRITE_BUFFER_LENGTH, "PART %s\n", channel);
-			addnewtimed(mystate->now, OER_TIMED_IRC_COMMAND, OER_TIMED_IRC_COMMAND_DUMMY, outstring2, NULL, NULL, NULL);
+			snprintf(timed_str, WRITE_BUFFER_LENGTH, "PART %s\n", channel);
+			timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_HANDLING, NULL, timed_str);
 		}
 		listchanusers();
 		return;
@@ -654,7 +657,7 @@ void parseirc(char *serveroutput)
 			modetype = nthmode(mode, i);
 			switch(modetype) {
 			case OER_NTHMODE_OP:
-				changeuser(channel, target, -1, 1, -1);
+				changeuser(channel, target, -1, 1, -1, -1);
 				if(isme(target)) {
 					iamop(channel, 1);
 					channelsync(channel);
@@ -669,21 +672,31 @@ void parseirc(char *serveroutput)
                                                 continue;
                                         }
 					oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->non-allowed op of nick %s on channel %s by %s!%s\n", target, channel, nick, userhost);
-					addnewmm(channel, mystate->now, "-o", target);
+					snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s -o %s\n", channel, target);
+					timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 				}
 				break;
 			case OER_NTHMODE_DOP:
-				changeuser(channel, target, -1, 0, -1);
+				changeuser(channel, target, -1, 0, -1, -1);
 				if(isme(target)) {
 					iamop(channel, 0);
 				}
+				if(isq(nick)) {
+					/* allow Q to freely de-op */
+					continue;
+				}
+				if(haschanflags(channel, "D") && isallowedop(channel, target)) {
+                                        /* re-op */
+					snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s +o %s\n", channel, target);
+					timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
+                                }
 				deops++;
 				break;
 			case OER_NTHMODE_VOICE:
-				changeuser(channel, target, -1, -1, 1);
+				changeuser(channel, target, -1, -1, 1, -1);
 				break;
 			case OER_NTHMODE_DVOICE:
-				changeuser(channel, target, -1, -1, 0);
+				changeuser(channel, target, -1, -1, 0, -1);
 				break;
 			case OER_NTHMODE_BAN:
 				snprintf(outstring, STRINGLEN, "%s!%s@%s", (mystate->use_alt_nick) ? mystate->altnick : mystate->nick, mystate->user, mystate->host);
@@ -696,10 +709,11 @@ void parseirc(char *serveroutput)
 					if(amiop(channel) && haschanflags(channel, "b")) {
 						/* i am being banned, first kick then unban */
 						if(!isme(nick) && !isserver(nick)) {
-							snprintf(outstring, STRINGLEN, "KICK %s %s :do NOT ban %s, that's me!\n", channel, nick, target);
-							addnewtimed(mystate->now, OER_TIMED_IRC_COMMAND, OER_TIMED_IRC_COMMAND_DUMMY, outstring, NULL, NULL, NULL);
+							snprintf(timed_str, WRITE_BUFFER_LENGTH, "KICK %s %s :do NOT ban %s, that's me!\n", channel, nick, target);
+							timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 						}
-						addnewmm(channel, mystate->now, "-b", target);
+						snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s -b %s\n", channel, target);
+						timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 					}
 				}
 				bans++;
@@ -712,7 +726,8 @@ void parseirc(char *serveroutput)
                                         }
 					/* re-set the ban */
 					oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->%s on channel %s tried to remove the permban %s\n", nick, channel, target);
-					addnewmm(channel, mystate->now, "+b", target);
+					snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s +b %s\n", channel, target);
+					timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 					if(!isserver(nick)) {
 						snprintf(outstring, STRINGLEN, "do NOT remove %s ban from %s, it is in my permban list", target, channel);
 						sendreply(nick, 0, 3, 0, outstring);
@@ -727,7 +742,8 @@ void parseirc(char *serveroutput)
                                         continue;
                                 }
 				if(amiop(channel) && haschanflags(channel, "m") && !isserver(nick)) {
-					addnewmm(channel, mystate->now, "-o", nick);
+					snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s -o %s\n", channel, nick);
+					timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 					sendreply(nick, 0, 3, 0, "do NOT deop >3 users at a time");
 					deops = 0;
 					bans = 0;
@@ -895,6 +911,7 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 	char uptimestring[STRINGLEN + 1];
 	char modeline[CHANLEN + 1];
 	char *restptr;
+	char *ptr;
 	struct channel *this;
 	struct chanuser *u;
 
@@ -1065,8 +1082,8 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 		if(!deltopics(channel)) {
 			return;
 		}
-		snprintf(outstring, WRITE_BUFFER_LENGTH, "TOPIC %s\n", channel);
-		addnewtimed(mystate->now, OER_TIMED_IRC_COMMAND, OER_TIMED_IRC_COMMAND_DUMMY, outstring, NULL, NULL, NULL);
+		snprintf(timed_str, WRITE_BUFFER_LENGTH, "TOPIC %s\n", channel);
+		timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_HANDLING, NULL, timed_str);
 		return;
 	case OER_WHICHCOMMAND_TOPIC_REFRESH:
 		oer_debug(OER_DEBUG_INFO, "parsecommand->TOPIC_REFRESH\n");
@@ -1254,7 +1271,8 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 		if(!strlen(restptr)) {
 			/* the messager wants to be opped */
 			if(!isopnow(channel, nick)) {
-				addnewmm(channel, mystate->now, "+o", nick);
+				snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s +o %s\n", channel, nick);
+				timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 			}
 			return;
 		}
@@ -1270,7 +1288,8 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 				if(haschanflags(channel, "u") && !isallowedop(channel, p1)) {
 					continue;
 				}
-				addnewmm(channel, mystate->now, "+o", p1);
+				snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s +o %s\n", channel, p1);
+				timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 			}
 		}
 		return;
@@ -1288,7 +1307,8 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 		if(!strlen(restptr)) {
 			/* the messager wants to be de-opped */
 			if(isopnow(channel, nick)) {
-				addnewmm(channel, mystate->now, "-o", nick);
+				snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s -o %s\n", channel, nick);
+				timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 			}
 			return;
 		}
@@ -1301,7 +1321,8 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 			}
 			ppos = nppos;
 			if(isopnow(channel, p1)) {
-				addnewmm(channel, mystate->now, "-o", p1);
+				snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s -o %s\n", channel, p1);
+				timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 			}
 		}
 		return;
@@ -1319,7 +1340,8 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 		if(!strlen(restptr)) {
 			/* the messager wants to be voiced */
 			if(!isvoicenow(channel, nick)) {
-				addnewmm(channel, mystate->now, "+v", nick);
+				snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s +v %s\n", channel, nick);
+				timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 			}
 			return;
 		}
@@ -1332,7 +1354,8 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 			}
 			ppos = nppos;
 			if(!isvoicenow(channel, p1)) {
-				addnewmm(channel, mystate->now, "+v", p1);
+				snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s +v %s\n", channel, p1);
+				timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 			}
 		}
 		return;
@@ -1442,8 +1465,8 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 			/* this makes no sense */
 			return;
 		}
-		snprintf(outstring, WRITE_BUFFER_LENGTH, "INVITE %s %s\n", nick, channel);
-		addnewtimed(mystate->now, OER_TIMED_IRC_COMMAND, OER_TIMED_IRC_COMMAND_DUMMY, outstring, NULL, NULL, NULL);
+		snprintf(timed_str, WRITE_BUFFER_LENGTH, "INVITE %s %s\n", nick, channel);
+		timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_HANDLING, NULL, timed_str);
 		return;
 	case OER_WHICHCOMMAND_MASSMESSAGE:
 		oer_debug(OER_DEBUG_INFO, "parsecommand->MASSMESSAGE\n");
@@ -1474,7 +1497,8 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 		}
 		if(!strlen(restptr + ppos)) {
 			/* only a mode line, no target */
-			addnewmm(channel, mystate->now, p1, "");
+			snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s %s\n", channel, p1);
+			timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 			return;
 		}
 		/* process all modes */
@@ -1487,7 +1511,8 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 			ppos = nppos;
 			getnthmode(p1, counter, modeline);
 			oer_debug(OER_DEBUG_INFO, "parsecommand->mode %s %s %s\n", channel, modeline, p2);
-			addnewmm(channel, mystate->now, modeline, p2);
+			snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s %s %s\n", channel, modeline, p2);
+			timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 			counter++;
 		}
 		return;
@@ -1669,10 +1694,11 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 		if(!isopa(userhost)) {
 			return;
 		}
-		if(strlen(restptr) > 0) {
-			snprintf(outstring, WRITE_BUFFER_LENGTH, "%s\n", restptr);
-                        addnewtimed(mystate->now, OER_TIMED_IRC_COMMAND, OER_TIMED_IRC_COMMAND_DUMMY, outstring, NULL, NULL, NULL);
+		if(!strlen(restptr)) {
+			return;
 		}
+		snprintf(timed_str, WRITE_BUFFER_LENGTH, "%s\n", restptr);
+		timed_new(mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, NULL, timed_str);
 		return;
 	case OER_WHICHCOMMAND_LAST:
 		oer_debug(OER_DEBUG_INFO, "parsecommand->LAST\n");
@@ -1741,6 +1767,26 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 		}
 		wall(channel, nick, userhost, restptr);
 		return;
+	case OER_WHICHCOMMAND_CLONECHANNEL:
+                oer_debug(OER_DEBUG_INFO, "parsecommand->CLONECHANNEL\n");
+                if(!isopa(userhost)) {
+                        return;
+                }
+                if(!strlen(restptr)) {
+                        return;
+                }
+                if((ptr = clonechannel(channel, restptr)) == NULL) {
+                        return;
+                }
+                snprintf(outstring, WRITE_BUFFER_LENGTH, "cloned channel %s to %s", channel, ptr);
+                sendreply((tochan) ? channel : nick, tochan, 0, 0, outstring);
+		if(!haschanflags(ptr, "L") && !haschanflags(ptr, "S")) {
+			return;
+		}
+		mysqldbname(outstring2, ptr, WRITE_BUFFER_LENGTH);
+		snprintf(outstring, WRITE_BUFFER_LENGTH, "database tables last_%s and seen_%s have to be created manually using the provided oer+MySQL-last.sql and oer+MySQL-seen.sql scripts", outstring2, outstring2);
+		sendreply((tochan) ? channel : nick, tochan, 0, 0, outstring);
+                return;
         case OER_WHICHCOMMAND_INVALID:
 		oer_debug(OER_DEBUG_INFO, "parsecommand->invalid or malformed command %s\n", command);
 		break;
