@@ -11,7 +11,8 @@
 CREATE TABLE admins (
   handle varchar(32) NOT NULL default '',
   flags varchar(32) default '',
-  ident varchar(64) NOT NULL default ''
+  ident varchar(64) NOT NULL default '',
+  KEY key1(handle, ident)
 ) TYPE=MyISAM;
 
 #
@@ -32,7 +33,8 @@ CREATE TABLE adverts (
 CREATE TABLE bantype (
   channel varchar(64) default '',
   type int(11) NOT NULL default '1',
-  ident varchar(64) NOT NULL default ''
+  ident varchar(64) NOT NULL default '',
+  KEY key1(channel, ident)
 ) TYPE=MyISAM;
 
 #
@@ -40,7 +42,7 @@ CREATE TABLE bantype (
 #
 
 CREATE TABLE banvars (
-  channel varchar(64) default '',
+  channel varchar(64) NOT NULL default '',
   b_auto_rejoin int(11) NOT NULL default '30',
   b_part_rejoin int(11) NOT NULL default '600',
   b_flood int(11) NOT NULL default '300',
@@ -48,7 +50,8 @@ CREATE TABLE banvars (
   b_bad_word int(11) NOT NULL default '3600',
   b_bad_nick int(11) NOT NULL default '3600',
   b_normal_ban int(11) NOT NULL default '3600',
-  ident varchar(64) NOT NULL default ''
+  ident varchar(64) NOT NULL default '',
+  KEY key1(channel, ident)
 ) TYPE=MyISAM;
 
 #
@@ -60,7 +63,8 @@ CREATE TABLE channels (
   chanmodes varchar(12) default '',
   chankey varchar(32) default '',
   chanflags varchar(32) default '',
-  ident varchar(64) NOT NULL default ''
+  ident varchar(64) NOT NULL default '',
+  KEY key1(name, ident)
 ) TYPE=MyISAM;
 
 #
@@ -78,7 +82,7 @@ CREATE TABLE conf (
   vhost varchar(64) default '',
   ident varchar(32) NOT NULL default '',
   PRIMARY KEY (ident),
-  KEY chanid(ident)
+  KEY key1(ident)
 ) TYPE=MyISAM;
 
 #
@@ -92,7 +96,8 @@ CREATE TABLE floodvars (
   f_interval int(11) NOT NULL default '60',
   f_lines int(11) NOT NULL default '10',
   f_chars int(11) NOT NULL default '1600',
-  ident varchar(64) NOT NULL default ''
+  ident varchar(64) NOT NULL default '',
+  KEY key1(channel, ident)
 ) TYPE=MyISAM;
 
 #
@@ -103,8 +108,9 @@ CREATE TABLE hostmasks (
   hostmask varchar(64) NOT NULL default '',
   channel varchar(64) default '',
   type int(11) NOT NULL default '0',
-  handle varchar(32) default '',
-  ident varchar(64) NOT NULL default ''
+  handle varchar(32) NOT NULL default '',
+  ident varchar(64) NOT NULL default '',
+  KEY key1(hostmask, channel, type, handle, ident)
 ) TYPE=MyISAM;
 
 #
@@ -113,7 +119,7 @@ CREATE TABLE hostmasks (
 
 CREATE TABLE kickreasons (
   reason varchar(255) NOT NULL default '',
-  ident varchar(64) NOT NULL default ''
+  ident varchar(4) NOT NULL default ''
 ) TYPE=MyISAM;
 
 #
@@ -126,7 +132,8 @@ CREATE TABLE nickbks (
   reason varchar(64) default '',
   setby varchar(32) default '',
   twhen int(10) unsigned NOT NULL default '0',
-  ident varchar(64) NOT NULL default ''
+  ident varchar(64) NOT NULL default '',
+  KEY key1(channel, pattern, ident)
 ) TYPE=MyISAM;
 
 #
@@ -144,11 +151,12 @@ CREATE TABLE output (
 #
 
 CREATE TABLE passwords (
-  handle varchar(32) default '',
-  channel varchar(64) default '',
+  handle varchar(32) NOT NULL default '',
+  channel varchar(64) NOT NULL default '',
   type int(11) NOT NULL default '0',
   password varchar(32) NOT NULL default '',
-  ident varchar(64) NOT NULL default ''
+  ident varchar(64) NOT NULL default '',
+  KEY key1(handle, channel, type, ident)
 ) TYPE=MyISAM;
 
 #
@@ -156,12 +164,13 @@ CREATE TABLE passwords (
 #
 
 CREATE TABLE permbans (
-  channel varchar(64) default '',
+  channel varchar(64) NOT NULL default '',
   mask varchar(64) NOT NULL default '',
   reason varchar(64) default '',
   setby varchar(32) default '',
   twhen int(10) unsigned NOT NULL default '0',
-  ident varchar(64) NOT NULL default ''
+  ident varchar(64) NOT NULL default '',
+  KEY key1(channel, mask, ident)
 ) TYPE=MyISAM;
 
 #
@@ -186,7 +195,8 @@ CREATE TABLE servers (
   protopers tinyint(4) NOT NULL default '0',
   linenoise tinyint(4) NOT NULL default '0',
   password varchar(32) default '',
-  ident varchar(32) NOT NULL default ''
+  ident varchar(32) NOT NULL default '',
+  KEY key1(hostname, port, servermodes, pingfreq, protopers, linenoise, ident)
 ) TYPE=MyISAM;
 
 #
@@ -196,8 +206,9 @@ CREATE TABLE servers (
 CREATE TABLE timestamps (
   ttype varchar(32) NOT NULL default '',
   twhen int(10) unsigned NOT NULL default '0',
-  tchannel varchar(32) default '',
-  ident varchar(64) NOT NULL default ''
+  tchannel varchar(32) NOT NULL default '',
+  ident varchar(64) NOT NULL default '',
+  KEY key1(ttype, tchannel, ident)
 ) TYPE=MyISAM;
 
 #
@@ -210,7 +221,8 @@ CREATE TABLE topics (
   twhen int(10) unsigned NOT NULL default '0',
   setby varchar(32) NOT NULL default '',
   message varchar(255) NOT NULL default '',
-  ident varchar(64) NOT NULL default ''
+  ident varchar(64) NOT NULL default '',
+  KEY key1(channel, pos, ident)
 ) TYPE=MyISAM;
 
 #
@@ -218,8 +230,9 @@ CREATE TABLE topics (
 #
 
 CREATE TABLE trusted (
-  hostmask varchar(64) NOT NULL default '',
-  ident varchar(64) NOT NULL default ''
+  host varchar(64) NOT NULL default '',
+  ident varchar(64) NOT NULL default '',
+  KEY key1(host, ident)
 ) TYPE=MyISAM;
 
 #
@@ -230,7 +243,8 @@ CREATE TABLE users (
   handle varchar(32) NOT NULL default '',
   channel varchar(64) default '',
   flags varchar(32) default '',
-  ident varchar(64) NOT NULL default ''
+  ident varchar(64) NOT NULL default '',
+  KEY key1(handle, channel, ident)
 ) TYPE=MyISAM;
 
 #
@@ -243,5 +257,9 @@ CREATE TABLE wordbks (
   reason varchar(64) default '',
   setby varchar(32) default '',
   twhen int(10) unsigned NOT NULL default '0',
-  ident varchar(64) NOT NULL default ''
+  ident varchar(64) NOT NULL default '',
+  KEY key1(channel, pattern, ident)
 ) TYPE=MyISAM;
+
+
+
